# @agnostack/requestd
![Test](https://github.com/agnostack/requestd/workflows/Test/badge.svg)

> 🎮 Minimal isomorphic Node or Browser request library.

## Installation

```bash
yarn add @agnostack/requestd
# npm install @agnostack/requestd
```

## Quickstart

```js
import { Client } from '@agnostack/requestd'
// const { Client } = require('@agnostack/requestd')

const requestd = new Client({
  store_url: '...', // Magento2 Store Url.
  username: '...', // Magento2 Admin username (for token).
  password: '...' // Magento2 Admin password (for token).
})

// const requestd = new Client({
//   store_url: '...', // Magento2 Store Url.
//   access_token: '...' // Magento2 Admin token.
// })

requestd
  .get('products/types')
  .then(console.log)
  .catch(console.error)

requestd
  .post('carts/12345/estimate-shipping-methods', {
    address: {
      country_id: 'US'
    }
  })
  .then(console.log)
  .catch(console.error)

requestd
  .put('/carts/12345', {
    customerId: 3,
    storeId: 1
  })
  .then(console.log)
  .catch(console.error)

// GraphQL
requestd
  .query({
    query: `
      {
        products(
          filter: { sku: { like: "sku-123" } }
        ) {
          items {
            sku
          }
        }
      }
    `
  })
  .then(console.log)
  .catch(console.error)
```

## Kitchen sink

```js
import MyCustomStorageAdapter from './MyCustomStorageAdapter'
const requestd = new Client({
  store_url: '...',
  username: '...',
  password: '...',
  access_token: '...',
  store_code: '...',
  storage: new MyCustomStorageAdapter()
  version: 'V1',
  headers: {
    // ...
  }
})
```

## Custom headers per request

The API provides you the ability to send various request headers that change the way data is stored or retrieved.

By default this library will encode all data as JSON, however you can customise this by setting your own `Content-Type` header as an additional argument to `get`, `post`, `put` and `delete`.

**Note**: If you add the `Content-Type` custom header to `post`, `put` or `delete` you will need to encode `data` yourself.

```js
const requestd = new Client({
  store_url: '...',
  username: '...',
  password: '...'
})

const headers = {
  'X-My-Header': 'custom'
}

requestd
  .get('products/types', headers)
  .then(console.log)
  .catch(console.error)
```

_Contact [Adam Grohs](https://www.linkedin.com/in/adamgrohs/) @ [agnoStack](https://agnostack.com/) for any questions._
